#include "drawlabel.h"
#include <qdebug.h>

DrawLabel::DrawLabel(QWidget *parent) : QLabel(parent)
{
    lines = new QLine[100];
    count = 0;
}


DrawLabel::~DrawLabel()
{
    delete lines;
}

void DrawLabel::paintEvent(QPaintEvent *)
{
    QPainter paint(this);
//    qDebug() << "update" << endl;
    QMatrix m;
    QLine invertedLine;
    QPoint point;
    paint.eraseRect(10,10,2*PLOT_WIDTH+10,2+PLOT_HEIGHT+10);

    paint.setPen(QColor(Qt::black));
    QPoint transOffs = QPoint(310,210);
    paint.translate(transOffs);

    // draw the axis
    // horizontal
    QLineF horAxis = QLineF(-PLOT_WIDTH,0,PLOT_WIDTH,0);
    paint.drawLine(horAxis);
    for (int i=1; i<10; i++) {
        QLineF horTicks = QLineF(60*i-PLOT_WIDTH,-4.0,60*i-PLOT_WIDTH,4.0);
        paint.drawLine(horTicks);
    }
    // vertical
    QLineF verAxis(0,-PLOT_HEIGHT,0,PLOT_HEIGHT);
    paint.drawLine(verAxis);
    for (int i=1; i<10; i++) {
        QLineF verTicks= QLineF(-4.0, 40*i-PLOT_HEIGHT, 4.0,40*i-PLOT_HEIGHT);
        paint.drawLine(verTicks);
   }
    // draw the lines
    paint.setPen(QColor(Qt::blue));
    //    paint.drawRect(-10,-10,20,20);
    for (int i=0; i<count;i++) {
        point = QPoint(lines[i].x1()*m_xscale,-lines[i].y1()*m_yscale);
        invertedLine.setP1(point);
        point = QPoint(lines[i].x2()*m_xscale,-lines[i].y2()*m_yscale);
        invertedLine.setP2(point);
        paint.drawLine(invertedLine);
//        qDebug() << "Draw Line x1 y1 x2 y2" << lines[i].x1()<<" " << lines[i].y1()<<
//                " " << lines[i].x2() << " " << lines[i].y2() << endl;
    }
    paint.setPen(QColor(Qt::red));
    if (enableEllipse) {
        m.rotate(angle);
        paint.setMatrix(m,true);
        paint.drawEllipse(ellipseRect);
        qDebug() << "Draw Ellipse x width" << ellipseRect.x()<<" " << ellipseRect.width()<< endl;
        qDebug() << "Draw Ellipse y height" << ellipseRect.y()<<" " << ellipseRect.height()<< endl;
    }
}
void DrawLabel::draw(QLine line)
{

    lines[count] = clip(line);
    count++;

    QLine parallel;
    QPoint point = QPoint(-line.x1(),-line.y1());
    parallel.setP1(point);
    point = QPoint(-line.x2(),-line.y2());
    parallel.setP2(point);
    lines[count] = clip(parallel);
    count++;

    update();
//    qDebug() << "count: " << count << endl;
}

QLine DrawLabel::clip(QLine line)
{
    double a,b,x1,y1,x2,y2;
    QLine clippedLine;
    double minX=PLOT_WIDTH/m_xscale;
    double minY=PLOT_WIDTH/m_yscale;

    x1 = (double)line.x1();
    x2 = (double)line.x2();
    y1 = (double)line.y1();
    y2 = (double)line.y2();

// determine the line equation
    a = (y2-y1)/(x2-x1);
    b = y1 - a*x1;

    x1 = -minX;
    y1 = x1*a + b;

    if (y1 < -minY) {
        y1 = -minY;
        x1 = (y1-b)/a;
    }
//    qDebug() << a << b << endl;

    if (y1 > minY) {
        y1 = minY;
        x1 = (y1-b)/a;
    }
    x2 = minX;
    y2 = x2 *a + b;
    if (y2 < -minY) {
        y2 = -minY;
        x2 = (y2-b)/a;
    }
    if (y2 > minY)  {
        y2 = minY;
        x2 = (y2-b)/a;
    }

    QPoint point = QPoint((int)x1,(int)y1);
    clippedLine.setP1(point);
    point = QPoint((int)x2,(int)y2);
    clippedLine.setP2(point);
//    qDebug() << " x1,y1,x2,y2 " <<x1<<","<<y1<<","<<x2<<","<<y2<<endl;
    return clippedLine;

}

